@@ -1,6 +1,6 @@
-use std::fmt;
-
 use crate::models::Frame;
+use serde::{Deserialize, Serialize};
+use std::fmt;
 
 /// Represents a genomic exon.
 ///
@@ -20,7 +20,7 @@ use crate::models::Frame;
 ///
 /// assert_eq!(coding_exon.is_coding(), true);
 /// ```
-#[derive(Debug, PartialEq, Eq)]
+#[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
 pub struct Exon {
     // u32 max value is 4,294,967,295 => This is sufficient for every human chromosome.
     // If you are working with species with chromsomes with more than 4 Mb per chromosome
